#include "stdafx.h"
#include "StateControl.h"
#include "NControl.h"
#include "RootContainer.h"
#include "AbstractButton.h"

CStateControl::CStateControl()
{
	m_pControl = NULL;
	m_eControlState = E_CONTROLSTATE_NORMAL;
	m_eControlStyle = E_STYLE_NORMAL;
	m_bPushed = FALSE;
}

CStateControl::~CStateControl()
{
}
CComponent* CStateControl::FindStatable(CComponent* target)
{
	CComponent* statableComponent = NULL;
	if (target == NULL) return statableComponent;
	CComponent* parent = target->GetParent();
	while (parent != NULL)
	{
		IStatable* statable = dynamic_cast<IStatable*>(parent);
		if (statable != NULL)
		{
			statableComponent = parent;
			break;
		}
		parent = parent->GetParent();
	}
	return statableComponent;
}
void CStateControl::AttachControl(CNControl* pControl)
{
	ATLASSERT(TRUE == IS_KIND_OF(CNControl, pControl));
	DetachControl();

	m_bPushed = FALSE;
	m_pControl = pControl;
	if (IS_KIND_OF(CAbstractButton, pControl))
	{
		CAbstractButton* pAbstractButton = dynamic_cast<CAbstractButton*>(pControl);
		if (pAbstractButton != NULL)
		{
			this->SetPushed(pAbstractButton->GetPushed());
			this->SetControlStyle(pAbstractButton->GetStyle() == CAbstractButton::E_BUTTON_STYLE_PUSHED ? E_STYLE_PUSH : E_STYLE_NORMAL);		
		}
	}
	m_pControl->AddMouseListener(dynamic_cast<IMouseListener*>(this));
	m_pControl->AddPropertyChangedListener(dynamic_cast<IPropertyChangedListener*>(this));
	
}

void CStateControl::DetachControl()
{
	if (m_pControl == NULL) return;

	m_pControl->RemoveMouseListener(dynamic_cast<IMouseListener*>(this));
	m_pControl->RemovePropertyChangedListener(dynamic_cast<IPropertyChangedListener*>(this));
	m_eControlState = E_CONTROLSTATE_NORMAL;
	m_eControlStyle = E_STYLE_NORMAL;
	m_bPushed = FALSE;
	m_pControl = NULL;
}

void CStateControl::OnMousePressed(CMouseEvent* pMouseEvent)
{
	if(pMouseEvent->m_nMouseButton != CMouseEvent::E_MOUSE_BUTTON_LEFT) return;

	m_eControlState = E_CONTROLSTATE_DOWN;
	
	m_pControl->Invalidate();
}

void CStateControl::OnMouseReleased(CMouseEvent* pMouseEvent)
{
	if(pMouseEvent->m_nMouseButton != CMouseEvent::E_MOUSE_BUTTON_LEFT) return;
	
	m_eControlState = E_CONTROLSTATE_NORMAL;

	m_pControl->Invalidate();
}

void CStateControl::OnMouseEntered(CMouseEvent* pMouseEvent)
{
	m_eControlState = E_CONTROLSTATE_NORMAL;
	m_pControl->Invalidate();
}

void CStateControl::OnMouseExited(CMouseEvent* pMouseEvent)
{
	m_eControlState = E_CONTROLSTATE_NORMAL;
	m_pControl->Invalidate();
}

void CStateControl::OnMouseMoved(CMouseEvent* pMouseEvent)
{
	CRootContainer* pRootContainer = m_pControl->GetRootContainer();
	if(pRootContainer->GetCapturedComponent() == m_pControl)
	{
		m_eControlState = E_CONTROLSTATE_DOWN;
	}
	else
	{
		m_eControlState = E_CONTROLSTATE_OVER;
	}
	m_pControl->Invalidate();
}

CStateControl::eControlState CStateControl::GetState()
{
	return m_eControlState;
}
void CStateControl::SetControlStyle(eControlStyle eStyle)
{
	m_eControlStyle = eStyle;
}
void CStateControl::SetPushed(BOOL bPushed)
{
	if (m_bPushed == bPushed) return;

	m_bPushed = bPushed;
}
void CStateControl::OnPropertyChanged(CPropertyChangeEvent* pPropertyChangedEvent)
{
	if (_tcscmp(pPropertyChangedEvent->m_szPropertyName, AbstractButtonProperty::Pushed) == 0)
	{
		CAbstractButton* button = dynamic_cast<CAbstractButton*>(pPropertyChangedEvent->m_pSource);
		if (button != NULL)
		{
			SetPushed(button->GetPushed());
		}		
	}
}